//
//  timers.c
//  Project - Digital Piano
//  Course  - Microprocessor Design
//
//  Created by Iulian Corcoja on 5/24/13.
//

// User defined headers 
#include "timers.h"

// Library defined scope variables
unsigned int _timer_value = 0;

// Init timers used for the piano
void init_timers(void) {

	// Enable global interrupts
	sei();

	// Set Timer0 for generating a 10ms (2HZ) interrupt on channel A
	TCCR0A = (1 << WGM01);					// Enable CTC
	TCCR0B = (1 << CS02) | (1 << CS00);		// Set prescaler to 1:1024
	OCR0A = 0x9C;							// Set count to generate the signal
	TIMSK0 |= (1 << OCIE0A);				// Enable timer interrupts

	// Set Timer1 for generating PWM signal on port D, pin 5
	TCCR1A |= _BV(WGM10);					// Enable PWN
	TCCR1B |= _BV(WGM13) | _BV(CS12);		// Set prescaler to 1:1024
	OCR1A = 59;								// Set count to generate the signal
}

// Interrupt function for Timer0
ISR(TIMER0_COMPA_vect) {
	int i;

	_timer_value = (_timer_value < tmr_max_value)? _timer_value + 1: 0;

	for (i = 0; i < tmr_count; i++) {
		if (_timer_value % tmr_call[i] == 0) {
			(*tmr_func_ptr[i])();
		}
	}
}
