//
//  utils.c
//  Project - Digital Piano
//  Course  - Microprocessor Design
//
//  Created by Iulian Corcoja on 5/24/13.
//

// C headers
#include <string.h>

// User defined headers 
#include "utils.h"

// Turns LED on
void led_on(volatile uint8_t *port, unsigned int pin) {
	*port |= _BV(pin);
}

// Turns LED off
void led_off(volatile uint8_t *port, unsigned int pin) {
	*port &= ~_BV(pin);
}

// Toggles LED on or off
void led_toggle(volatile uint8_t *port, unsigned int pin) {
	*port ^= _BV(pin);
}

// Returns 1 if push button is pressed
unsigned int btn_pushed(volatile uint8_t *port, unsigned int pin) {
	if (!(*port & _BV(pin)))
		return 1;

	return 0;
}

// Sets PWM oscilation frquency for Timer1
void set_pwm_freq(uint8_t freq) {
	OCR1A = freq;
}

// Turns PWM on for Timer1
void pwm_on(void) {
	TCCR1A |= _BV(COM1A0);
}

// Turns PWM on for Timer1
void pwm_off(void) {
	TCCR1A &= ~_BV(COM1A0);
}

// Init data for recording
void init_recording(void) {

	// Erase song memory
	memset(last_song, 0, MAX_SONG_SIZE * sizeof(song_note));

	// Set recording position
	rec_pos = 0;

	// Same init as playback
	init_playback();
}

// Init data for playback
void init_playback(void) {
	play_pos = 0;
	last_note_freq = 0;
	last_note_duration = 0;
}

// Record a song
int8_t record_song(uint8_t freq) {
	// Increment the duration of current note
	last_song[rec_pos].duration++;

	// New note has been registered
	if (freq != last_song[rec_pos].note_freq) {
		if (rec_pos + 1 >= MAX_SONG_SIZE)
			return -1;

		// Next note
		rec_pos++;

		// Init new note
		last_song[rec_pos].note_freq = freq;
		last_song[rec_pos].duration = 0;
	}

	// Return with success
	return 0;
}

// Play last recorded a song
int8_t playback_song(int *freq) {
	// Increment the duration of current note
	last_note_duration++;

	// Duration of the curent note ended
	if (last_note_duration >= last_song[play_pos].duration) {
		last_note_duration = 0;
		play_pos++;
	}

	// Song ended
	if (play_pos > rec_pos || play_pos >= MAX_SONG_SIZE)
		return -1;

	// Set the note PWM frequency
	*freq = last_song[play_pos].note_freq;

	// Return with success
	return 0;
}
