//
//  utils.h
//  Project - Digital Piano
//  Course  - Microprocessor Design
//
//  Created by Iulian Corcoja on 5/24/13.
//

#ifndef UTILS_
#define UTILS_

// AVR headers
#include <avr/io.h>

// Define LEDs by color
#define GREEN_LED	7
#define RED_LED		6

// Maximum song size in notes
#define MAX_SONG_SIZE 512

// How many milliseconds * 10 should pass until another state can be changed
#define STATE_TIMEOUT 50

// Define a note type
typedef struct {
	uint8_t note_freq;
	uint16_t duration;
} song_note; 

// Define external variables
extern song_note last_song[MAX_SONG_SIZE];
extern unsigned int rec_pos;
extern unsigned int play_pos;
extern uint8_t last_note_freq;
extern uint16_t last_note_duration;

// Turns LED on
void led_on(volatile uint8_t *port, unsigned int pin);

// Turns LED off
void led_off(volatile uint8_t *port, unsigned int pin);

// Toggles LED on or off
void led_toggle(volatile uint8_t *port, unsigned int pin);

// Returns 1 if push button is pressed
unsigned int btn_pushed(volatile uint8_t *port, unsigned int pin);

// Sets PWM oscilation frquency for Timer1
void set_pwm_freq(uint8_t freq);

// Turns PWM on for Timer1
void pwm_on(void);

// Turns PWM on for Timer1
void pwm_off(void);

// Init data for recording
void init_recording(void);

// Init data for playback
void init_playback(void);

// Record a song
int8_t record_song(uint8_t freq);

// Play last recorded a song
int8_t playback_song(int *freq);

#endif
